#include "codes.h"
#include "classes.h"

CCpu::CCpu(CMemory *pmArg,UI32 u32Arg) {

	pmMemory=pmArg;

	GR_LAST_INDEX=7;
	REG_SP=0x0d;
	REG_SR=0x0e;
	REG_PC=0x0f;

	SR_ZERO=0x00000001;
	SR_SIG= 0x00000002;
	SR_OF=  0x00000004;

	reset();

	setPC(u32Arg);

}

bool CCpu::reset() {

	for(int i=0;i<16;i++)
		u32Reg[i]=0;

	return true;

};

UI32 CCpu::getRegister(I32 i32Arg) const {

	if (i32Arg<0 || i32Arg>GR_LAST_INDEX)
		throw out_of_range("ȃWX^w");

	return u32Reg[i32Arg];

}

bool CCpu::setRegister(I32 i32Arg,UI32 u32Arg) {

	if (i32Arg<0 || i32Arg>GR_LAST_INDEX)
		throw out_of_range("ȃWX^w");

	u32Reg[i32Arg]=u32Arg;

	return true;

}

UI32 CCpu::getPC() const { return u32Reg[REG_PC]; }
UI32 CCpu::getSP() const { return u32Reg[REG_SP]; }
UI32 CCpu::getSR() const { return u32Reg[REG_SR]; }

bool CCpu::setPC(UI32 u32Arg) {

	u32Reg[REG_PC]=u32Arg;

	return true;

}

bool CCpu::setSP(UI32 u32Arg) {

	u32Reg[REG_SP]=u32Arg;

	return true;

}

bool CCpu::setSR(UI32 u32Arg) {

	u32Reg[REG_SR]=u32Arg;

	return true;

}

bool CCpu::getSRZero() const { return (u32Reg[REG_SR] & SR_ZERO!=0); }
bool CCpu::getSRSig() const { return (u32Reg[REG_SR] & SR_SIG!=0); }
bool CCpu::getSROF() const { return (u32Reg[REG_SR] & SR_OF!=0); }

bool CCpu::exec() throw(logic_error) {

	exec(getPC());

	return true;

}

bool CCpu::exec(UI32 u32Arg) {

	UI32 u32Code=pmMemory->read32(u32Arg);
	UI32 u32Wrk1,u32Wrk2,u32Addr;
	UI16 u16Wrk1,u16Wrk2;
	UI8 u8Wrk1,u8Wrk2;

	switch ((u32Code & 0xffff0000) >> 16) {

	case LDRR:

		u32Wrk1=(u32Code & 0xff00) >> 8;
		u32Wrk2=u32Code & 0x00ff;

		setRegister(u32Wrk1,getRegister(u32Wrk2));

		u32Addr=u32Arg+4;

		break;

	case LDRI8:

		u32Wrk1=(u32Code & 0xff00) >> 8;

		setRegister(u32Wrk1,u32Code & 0x00ff);

		u32Addr=u32Arg+4;

		break;

	case LDRI16:

		u32Wrk1=(u32Code & 0xff00) >> 8;
		u32Wrk2=pmMemory->read16(u32Arg+4);

		setRegister(u32Wrk1,u32Wrk2);

		u32Addr=u32Arg+6;

		break;

	case LDRI32:

		u32Wrk1=(u32Code & 0xff00) >> 8;
		u32Wrk2=pmMemory->read32(u32Arg+4);

		setRegister(u32Wrk1,u32Wrk2);

		u32Addr=u32Arg+8;

		break;

	case LDRM8:

		u32Wrk1=(u32Code & 0xff00) >> 8;
		u8Wrk1=pmMemory->read8(getRegister(u32Code & 0xff));

		setRegister(u32Wrk1,u8Wrk1);

		u32Addr=u32Arg+4;

		break;

	case LDRM16:

		u32Wrk1=(u32Code & 0xff00) >> 8;
		u16Wrk1=pmMemory->read16(getRegister(u32Code & 0xff));

		setRegister(u32Wrk1,u16Wrk1);

		u32Addr=u32Arg+4;

		break;

	case LDRM32:

		u32Wrk1=(u32Code & 0xff00) >> 8;
		u32Wrk2=pmMemory->read32(getRegister(u32Code & 0xff));

		setRegister(u32Wrk1,u32Wrk2);

		u32Addr=u32Arg+4;

		break;

	case LDMR8:

		u32Wrk1=(u32Code & 0xff00) >> 8;
		u8Wrk1=(UI8)(getRegister(u32Code & 0xff) & 0x00ff);

		pmMemory->write8(getRegister(u32Wrk1),u8Wrk1);

		u32Addr=u32Arg+4;

		break;

	case LDMR16:

		u32Wrk1=(u32Code & 0xff00) >> 8;
		u16Wrk1=(UI16)(getRegister(u32Code & 0xff) & 0xffff);

		pmMemory->write16(getRegister(u32Wrk1),u16Wrk1);

		u32Addr=u32Arg+4;

		break;

	case LDMR32:

		u32Wrk1=(u32Code & 0xff00) >> 8;
		u32Wrk2=getRegister(u32Code & 0xff);

		pmMemory->write32(getRegister(u32Wrk1),u32Wrk2);

		u32Addr=u32Arg+4;

		break;

	case LDMI8:

		u32Wrk1=(u32Code & 0xff00) >> 8;
		u8Wrk2=(UI8)(u32Code & 0xff);

		pmMemory->write8(getRegister(u32Wrk1),u8Wrk2);

		u32Addr=u32Arg+4;

		break;

	case LDMI16:

		u32Wrk1=(u32Code & 0xff00) >> 8;
		u16Wrk2=pmMemory->read16(u32Arg+4);

		pmMemory->write16(getRegister(u32Wrk1),u8Wrk2);

		u32Addr=u32Arg+6;

		break;

	case LDMI32:

		u32Wrk1=(u32Code & 0xff00) >> 8;
		u32Wrk2=pmMemory->read32(u32Arg+4);

		pmMemory->write32(getRegister(u32Wrk1),u32Wrk2);

		u32Addr=u32Arg+8;

		break;

	case EXC:

		u32Wrk1=getRegister((u32Code & 0xff00) >> 8);
		setRegister((u32Code & 0xff00) >> 8,getRegister(u32Code & 0xff));
		setRegister(u32Code & 0xff,u32Wrk1);

		u32Addr=u32Arg+4;

		break;

	default:

		throw invalid_argument("ȃR[h");

	}

	setPC(u32Addr);

	return true;

}

CCpu::~CCpu() { }
